/*
 / _____)             _              | |
( (____  _____ ____ _| |_ _____  ____| |__
 \____ \| ___ |    (_   _) ___ |/ ___)  _ \
 _____) ) ____| | | || |_| ____( (___| | | |
(______/|_____)_|_|_| \__)_____)\____)_| |_|
    (C)2014 Semtech

Description: State Grid CRC and Whitening Method

License: Revised BSD License, see LICENSE.TXT file include in the project

Maintainer: Jiapeng Li
*/

#include "sx1208-state-grid.h"

/** whitening option, 1: enable, 0: disable, others: error */
#define SX1208_STATE_GRID_WHITENING_ENABLE			1

/** bit stream reverse option, 1: enable, 0: disable, others: error */
#define SX1208_STATE_GRID_REVERSE_ENABLE			1

#define CRC_POLY 0x8408
uint16_t sx1208_state_grid_crc_calc(uint8_t *buf, uint8_t len)
{
	uint16_t crc=0xffff;
    uint8_t i;

	/** first byte is payload length */
	crc^=len;
	for(i=0;i<8;i++){
		if (crc&0x0001){
			crc >>= 1;
			crc^= CRC_POLY;
		}else{
			crc >>= 1;
		}
	}

    while(len--){
        crc^=*buf++;
        for(i=0;i<8;i++){
            if (crc&0x0001){
                crc >>= 1;
                crc^= CRC_POLY;
            }else{
                crc >>= 1;
            }
        }
    }
    crc ^= 0xffff;
    return crc;
}

uint16_t sx1208_state_grid_crc_check(uint8_t *buf, uint8_t len)
{
	uint16_t crc=0xffff;
    uint8_t i;

	/** first byte is payload length */
	crc^=len;
	for(i=0;i<8;i++){
		if (crc&0x0001){
			crc >>= 1;
			crc^= CRC_POLY;
		}else{
			crc >>= 1;
		}
	}

	/** include crc bytes */
	len += 2;

    while(len--){
        crc^=*buf++;
        for(i=0;i<8;i++){
            if (crc&0x0001){
                crc >>= 1;
                crc^= CRC_POLY;
            }else{
                crc >>= 1;
            }
        }
    }
    crc ^= 0xffff;
    return crc;
}

static const uint8_t sx1208_whitening_tab[256] = {
	0xFF, 0xE1, 0x1D, 0x9A, 0xED, 0x85, 0x33, 0x24,
	0xEA, 0x7A, 0xD2, 0x39, 0x70, 0x97, 0x57, 0x0A,
	0x54, 0x7D, 0x2D, 0xD8, 0x6D, 0x0D, 0xBA, 0x8F,
	0x67, 0x59, 0xC7, 0xA2, 0xBF, 0x34, 0xCA, 0x18,
	0x30, 0x53, 0x93, 0xDF, 0x92, 0xEC, 0xA7, 0x15,
	0x8A, 0xDC, 0xF4, 0x86, 0x55, 0x4E, 0x18, 0x21,
	0x40, 0xC4, 0xC4, 0xD5, 0xC6, 0x91, 0x8A, 0xCD,
	0xE7, 0xD1, 0x4E, 0x09, 0x32, 0x17, 0xDF, 0x83,
	0xFF, 0xF0, 0x0E, 0xCD, 0xF6, 0xC2, 0x19, 0x12,
	0x75, 0x3D, 0xE9, 0x1C, 0xB8, 0xCB, 0x2B, 0x05,
	0xAA, 0xBE, 0x16, 0xEC, 0xB6, 0x06, 0xDD, 0xC7,
	0xB3, 0xAC, 0x63, 0xD1, 0x5F, 0x1A, 0x65, 0x0C,
	0x98, 0xA9, 0xC9, 0x6F, 0x49, 0xF6, 0xD3, 0x0A,
	0x45, 0x6E, 0x7A, 0xC3, 0x2A, 0x27, 0x8C, 0x10,
	0x20, 0x62, 0xE2, 0x6A, 0xE3, 0x48, 0xC5, 0xE6,
	0xF3, 0x68, 0xA7, 0x04, 0x99, 0x8B, 0xEF, 0xC1,
	0x7F, 0x78, 0x87, 0x66, 0x7B, 0xE1, 0x0C, 0x89,
	0xBA, 0x9E, 0x74, 0x0E, 0xDC, 0xE5, 0x95, 0x02,
	0x55, 0x5F, 0x0B, 0x76, 0x5B, 0x83, 0xEE, 0xE3,
	0x59, 0xD6, 0xB1, 0xE8, 0x2F, 0x8D, 0x32, 0x06,
	0xCC, 0xD4, 0xE4, 0xB7, 0x24, 0xFB, 0x69, 0x85,
	0x22, 0x37, 0xBD, 0x61, 0x95, 0x13, 0x46, 0x08,
	0x10, 0x31, 0x71, 0xB5, 0x71, 0xA4, 0x62, 0xF3,
	0x79, 0xB4, 0x53, 0x82, 0xCC, 0xC5, 0xF7, 0xE0,
	0x3F, 0xBC, 0x43, 0xB3, 0xBD, 0x70, 0x86, 0x44,
	0x5D, 0x4F, 0x3A, 0x07, 0xEE, 0xF2, 0x4A, 0x81,
	0xAA, 0xAF, 0x05, 0xBB, 0xAD, 0x41, 0xF7, 0xF1,
	0x2C, 0xEB, 0x58, 0xF4, 0x97, 0x46, 0x19, 0x03,
	0x66, 0x6A, 0xF2, 0x5B, 0x92, 0xFD, 0xB4, 0x42,
	0x91, 0x9B, 0xDE, 0xB0, 0xCA, 0x09, 0x23, 0x04,
	0x88, 0x98, 0xB8, 0xDA, 0x38, 0x52, 0xB1, 0xF9,
	0x3C, 0xDA, 0x29, 0x41, 0xE6, 0xE2, 0x7B, 0xF0,
};

//uint8_t sx1208_state_grid_whitening_data(uint8_t data, uint8_t index)
//{
//	return (data^sx1208_whitening_tab[index]);
//}
//
//void sx1208_state_grid_whitening_buf(uint8_t *buf, uint16_t len, uint8_t start_index)
//{
//	uint16_t i;
//
//	for(i=0; i<len; i++){
//		buf[i] ^= sx1208_whitening_tab[start_index+i];
//	}
//}

static const uint8_t sx1208_reverse_tab[256] = {
	0x00, 0x80, 0x40, 0xC0, 0x20, 0xA0, 0x60, 0xE0,
	0x10, 0x90, 0x50, 0xD0, 0x30, 0xB0, 0x70, 0xF0,
	0x08, 0x88, 0x48, 0xC8, 0x28, 0xA8, 0x68, 0xE8,
	0x18, 0x98, 0x58, 0xD8, 0x38, 0xB8, 0x78, 0xF8,
	0x04, 0x84, 0x44, 0xC4, 0x24, 0xA4, 0x64, 0xE4,
	0x14, 0x94, 0x54, 0xD4, 0x34, 0xB4, 0x74, 0xF4,
	0x0C, 0x8C, 0x4C, 0xCC, 0x2C, 0xAC, 0x6C, 0xEC,
	0x1C, 0x9C, 0x5C, 0xDC, 0x3C, 0xBC, 0x7C, 0xFC,
	0x02, 0x82, 0x42, 0xC2, 0x22, 0xA2, 0x62, 0xE2,
	0x12, 0x92, 0x52, 0xD2, 0x32, 0xB2, 0x72, 0xF2,
	0x0A, 0x8A, 0x4A, 0xCA, 0x2A, 0xAA, 0x6A, 0xEA,
	0x1A, 0x9A, 0x5A, 0xDA, 0x3A, 0xBA, 0x7A, 0xFA,
	0x06, 0x86, 0x46, 0xC6, 0x26, 0xA6, 0x66, 0xE6,
	0x16, 0x96, 0x56, 0xD6, 0x36, 0xB6, 0x76, 0xF6,
	0x0E, 0x8E, 0x4E, 0xCE, 0x2E, 0xAE, 0x6E, 0xEE,
	0x1E, 0x9E, 0x5E, 0xDE, 0x3E, 0xBE, 0x7E, 0xFE,
	0x01, 0x81, 0x41, 0xC1, 0x21, 0xA1, 0x61, 0xE1,
	0x11, 0x91, 0x51, 0xD1, 0x31, 0xB1, 0x71, 0xF1,
	0x09, 0x89, 0x49, 0xC9, 0x29, 0xA9, 0x69, 0xE9,
	0x19, 0x99, 0x59, 0xD9, 0x39, 0xB9, 0x79, 0xF9,
	0x05, 0x85, 0x45, 0xC5, 0x25, 0xA5, 0x65, 0xE5,
	0x15, 0x95, 0x55, 0xD5, 0x35, 0xB5, 0x75, 0xF5,
	0x0D, 0x8D, 0x4D, 0xCD, 0x2D, 0xAD, 0x6D, 0xED,
	0x1D, 0x9D, 0x5D, 0xDD, 0x3D, 0xBD, 0x7D, 0xFD,
	0x03, 0x83, 0x43, 0xC3, 0x23, 0xA3, 0x63, 0xE3,
	0x13, 0x93, 0x53, 0xD3, 0x33, 0xB3, 0x73, 0xF3,
	0x0B, 0x8B, 0x4B, 0xCB, 0x2B, 0xAB, 0x6B, 0xEB,
	0x1B, 0x9B, 0x5B, 0xDB, 0x3B, 0xBB, 0x7B, 0xFB,
	0x07, 0x87, 0x47, 0xC7, 0x27, 0xA7, 0x67, 0xE7,
	0x17, 0x97, 0x57, 0xD7, 0x37, 0xB7, 0x77, 0xF7,
	0x0F, 0x8F, 0x4F, 0xCF, 0x2F, 0xAF, 0x6F, 0xEF,
	0x1F, 0x9F, 0x5F, 0xDF, 0x3F, 0xBF, 0x7F, 0xFF,
};

// TX: Whitening First, Then Reverse
uint8_t sx1208_state_grid_whitening_reverse_data(uint8_t data, uint8_t index)
{
#if ( (SX1208_STATE_GRID_WHITENING_ENABLE == 1) && (SX1208_STATE_GRID_REVERSE_ENABLE == 1) )

	/** WHITENING and REVERSE are both enabled */
	return sx1208_reverse_tab[(data^sx1208_whitening_tab[index])];

#elif ( (SX1208_STATE_GRID_WHITENING_ENABLE == 1) && (SX1208_STATE_GRID_REVERSE_ENABLE == 0) )

	/** WHITENING is enabled, REVERSE is disabled */
	return (data^sx1208_whitening_tab[index]);

#elif ( (SX1208_STATE_GRID_WHITENING_ENABLE == 0) && (SX1208_STATE_GRID_REVERSE_ENABLE == 1) )

	/** WHITENING is disabled, REVERSE is enabled */
	return sx1208_reverse_tab[data];

#elif ( (SX1208_STATE_GRID_WHITENING_ENABLE == 0) && (SX1208_STATE_GRID_REVERSE_ENABLE == 0) )

	/** WHITENING and REVERSE are both disabled */
	return data;

#else
	#error "WHITENING and REVERSE option is wrong."
#endif
}

void sx1208_state_grid_whitening_reverse_buf(uint8_t *buf, uint16_t len, uint8_t start_index)
{
	uint16_t i;

	for(i=0; i<len; i++){
#if ( (SX1208_STATE_GRID_WHITENING_ENABLE == 1) && (SX1208_STATE_GRID_REVERSE_ENABLE == 1) )

		/** WHITENING and REVERSE are both enabled */
		buf[i] ^= sx1208_whitening_tab[start_index+i];
		buf[i] = sx1208_reverse_tab[buf[i]];

#elif ( (SX1208_STATE_GRID_WHITENING_ENABLE == 1) && (SX1208_STATE_GRID_REVERSE_ENABLE == 0) )

		/** WHITENING is enabled, REVERSE is disabled */
		buf[i] ^= sx1208_whitening_tab[start_index+i];

#elif ( (SX1208_STATE_GRID_WHITENING_ENABLE == 0) && (SX1208_STATE_GRID_REVERSE_ENABLE == 1) )

		/** WHITENING is disabled, REVERSE is enabled */
		buf[i] = sx1208_reverse_tab[buf[i]];

#elif ( (SX1208_STATE_GRID_WHITENING_ENABLE == 0) && (SX1208_STATE_GRID_REVERSE_ENABLE == 0) )

		/** WHITENING and REVERSE are both disabled */
		// do nothing

#else
	#error "WHITENING and REVERSE option is wrong."
#endif
	}
}

// RX: Reverse First, Then Whitening
uint8_t sx1208_state_grid_reverse_whitening_data(uint8_t data, uint8_t index)
{
#if ( (SX1208_STATE_GRID_WHITENING_ENABLE == 1) && (SX1208_STATE_GRID_REVERSE_ENABLE == 1) )

	/** WHITENING and REVERSE are both enabled */
	return (sx1208_reverse_tab[data]^sx1208_whitening_tab[index]);

#elif ( (SX1208_STATE_GRID_WHITENING_ENABLE == 1) && (SX1208_STATE_GRID_REVERSE_ENABLE == 0) )

	/** WHITENING is enabled, REVERSE is disabled */
	return (data^sx1208_whitening_tab[index]);

#elif ( (SX1208_STATE_GRID_WHITENING_ENABLE == 0) && (SX1208_STATE_GRID_REVERSE_ENABLE == 1) )

	/** WHITENING is disabled, REVERSE is enabled */
	return (sx1208_reverse_tab[data]);

#elif ( (SX1208_STATE_GRID_WHITENING_ENABLE == 0) && (SX1208_STATE_GRID_REVERSE_ENABLE == 0) )

	/** WHITENING and REVERSE are both disabled */
	return (data);

#else
	#error "WHITENING and REVERSE option is wrong."
#endif
}

void sx1208_state_grid_reverse_whitening_buf(uint8_t *buf, uint16_t len, uint8_t start_index)
{
	uint16_t i;

	for(i=0; i<len; i++){
#if ( (SX1208_STATE_GRID_WHITENING_ENABLE == 1) && (SX1208_STATE_GRID_REVERSE_ENABLE == 1) )

		/** WHITENING and REVERSE are both enabled */
		buf[i] = sx1208_reverse_tab[buf[i]];
		buf[i] ^= sx1208_whitening_tab[start_index+i];

#elif ( (SX1208_STATE_GRID_WHITENING_ENABLE == 1) && (SX1208_STATE_GRID_REVERSE_ENABLE == 0) )

		/** WHITENING is enabled, REVERSE is disabled */
		buf[i] ^= sx1208_whitening_tab[start_index+i];

#elif ( (SX1208_STATE_GRID_WHITENING_ENABLE == 0) && (SX1208_STATE_GRID_REVERSE_ENABLE == 1) )

		/** WHITENING is disabled, REVERSE is enabled */
		buf[i] = sx1208_reverse_tab[buf[i]];

#elif ( (SX1208_STATE_GRID_WHITENING_ENABLE == 0) && (SX1208_STATE_GRID_REVERSE_ENABLE == 0) )

		/** WHITENING and REVERSE are both disabled */
		// do nothing

#else
	#error "WHITENING and REVERSE option is wrong."
#endif
	}
}

uint8_t sx1208_state_grid_reverse(uint8_t val)
{
#if ( (SX1208_STATE_GRID_WHITENING_ENABLE == 1) && (SX1208_STATE_GRID_REVERSE_ENABLE == 1) )

	/** WHITENING and REVERSE are both enabled */
	return sx1208_reverse_tab[val];

#elif ( (SX1208_STATE_GRID_WHITENING_ENABLE == 1) && (SX1208_STATE_GRID_REVERSE_ENABLE == 0) )

	/** WHITENING is enabled, REVERSE is disabled */
	return val;

#elif ( (SX1208_STATE_GRID_WHITENING_ENABLE == 0) && (SX1208_STATE_GRID_REVERSE_ENABLE == 1) )

	/** WHITENING is disabled, REVERSE is enabled */
	return sx1208_reverse_tab[val];

#elif ( (SX1208_STATE_GRID_WHITENING_ENABLE == 0) && (SX1208_STATE_GRID_REVERSE_ENABLE == 0) )

	/** WHITENING and REVERSE are both disabled */
	return val;

#else
	#error "WHITENING and REVERSE option is wrong."
#endif
}

